﻿--[[
    Copyright Daniel D. Neilson, 2006

    This file is part of DKPmon.

    DKPmon is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DKPmon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DKPmon; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
]]
--[[
    Functions for setting up & performing bidding-related operations.
    
    Data fields:
      DKPmon.bidding.bidstate:  number, which state of bidding are we in
            0 -- Building the list of bidable items
            1 -- Bidding is open, and we're accepting bids
            2 -- Bidding is closed and we're deciding winners
            3 -- Winners have been announced, waiting for point deduction
      DKPmon.bidding.closeTime: number; amount of time remaining 'til bidding's closed
      DKPmon.bidding.nitems: number; number of items in the list of bidable items
      DKPmon.bidding[i]: Table; information about bidable item i
        {
            link=<itemlink>,
            name=<name of item>,
            colour=<hex colour code of the item>,
            cost=<point cost of the item>,
            pool=<pool the item falls in>,
            isdistributed=<boolean; true iff the item has been distributed via the "ML" button on the GUI>,
            nbidders = 1+<number of bidders on the item>,
            winner = <string: name of the item winner>,
            bidders = {[0]={name="No Winner"}, [i>0]={name=<name of a sender>, bidder=<bidding name>, class=<class of sender>}}
        }

]]

local dewdrop = AceLibrary("Dewdrop-2.0")
    
local dewOptions = {}

--[[
    DKPmon:InitBiddingFields()
    Description:
        Initializes the data table: self.bidding
    Input:
        None
    Output:
        None
]]
function DKPmon:InitBiddingFields()
    if self.bidding ~= nil then self:TableDelete(self.bidding) end
    self.bidding = { bidstate = 0, closeTime = 0, nitems = 0 }
end

--[[
    DKPmon:ClearBidItems()
    Description:
        Wipes the biditems list of all items.
    Input:
        None
    Output:
        None
]]
function DKPmon:ClearBidItems()
    if self.bidding.bidstate > 0 then
        -- Bidding's active, have to confirm that the clear should go through
        if StaticPopupDialogs["DKPMON_CONFIRM_CLEAR"] == nil then
            self:CreateConfirmClearDialog()
        end
        StaticPopup_Show("DKPMON_CONFIRM_CLEAR")
        return
    end
    self:ConfirmedClearBidItems()
end

--[[
    DKPmon:ConfirmedClearBidItems()
    Description:
        Called once we've confirmed that the list of biditems should be cleared.
        Wipes the biditems list of all items.
    Input:
        None
    Output:
        None
]]
function DKPmon:ConfirmedClearBidItems()
    if self.bidding.bidstate > 0 then
        -- Bidding's active, have to close it down
        self:StopMetro("DKPmonClose") -- Stop the timer if one's going
        self:SendCloseBidding() -- Send the broadcast to stop bidding
        SendChatMessage("Sorry, bidding was cancelled.", "RAID")
    end
    self:InitBiddingFields()
    self:ResetBidUI()
end

--[[
    DKPmon:PlaceBid(itemID, bidname)
    Description:
        Places a bid for the item self.bidding[itemID] under the name bidname
    Input:
        itemID: The index of the item the bid's for in the item list
        bidname: The bidding name to bid under
	sender: The character name of the person who sent this bid
        senderclass: Class name, in english, of the sender
        bid: 'Y' = place a bid on the item, 'N' = remove bid on the item
    Output:
        boolean: true = bid added, false = bid removed
]]
function DKPmon:PlaceBid(itemID, bidname, sender, senderclass, bid)
    -- Make sure itemID is a valid ID
    if not (itemID < self.bidding.nitems) then return end
    -- Check if the person's already in the bidlist.
    local i = 0 
    --self:Print("Adding bidder "..bidname.." to itemID "..itemID)
    --self:Print("nbidders = "..self.bidding[itemID].nbidders)
    
    while i ~= self.bidding[itemID].nbidders do
            --self:Print(i)
        if self.bidding[itemID].bidders[i].name == sender then 
            -- We've found the name
            
            -- If they still want to bid on the item (bid='Y'), then we're done.
            if bid == "Y" then 
                -- Set the bidding name, just in case it's different
                self.bidding[itemID].bidders[i].bidder = bidname
                return true 
            end 
            
            -- They don't want to bid on the item anymore. So, we have to remove them from the list
            self.bidding[itemID].nbidders = self.bidding[itemID].nbidders - 1
            local pos = self.bidding[itemID].nbidders 
            -- Move the name at the end of the list into the position of this bidder.
            self:TableDelete(self.bidding[itemID].bidders[i])
            self.bidding[itemID].bidders[i] = self.bidding[itemID].bidders[pos]
            self.bidding[itemID].bidders[pos] = nil

            -- Update the GUI
            self:UpdateBidLine(itemID)
            return false
        end
        i = i + 1
    end
    -- Person wasn't in the bid list.
    
    -- If they don't want to bid on the item (bid = 'N') then we're done
    if bid == 'N' then return end
    local tab = {}
    tab.name = sender
    tab.bidder = bidname
    tab.class = senderclass
    self.bidding[itemID].bidders[ self.bidding[itemID].nbidders ] = tab
    self.bidding[itemID].nbidders = self.bidding[itemID].nbidders + 1
    self:UpdateBidLine(itemID)
    return true
end

--[[
    DKPmon:AddBidItemToList(link, name, colour, cost, pool)
    Description:
        Adds an item to bid on to the list of bidable items.
    Input:
        link: The item link for the item
        name: Name of the item
        colour: Colour of the item, as a hexcode (purple=epic, etc)
        cost: Point cost of the item
        pool: Pool from which the item draws points.
    Output:
        None
]]
function DKPmon:AddBidItemToList(link, name, colour, cost, pool)
    if self.bidding.nitems == 10 then
        self:Print("Cannot have more than 10 bidable items at a time")
        return
    end
    self.bidding[self.bidding.nitems] = 
	{ link = link, name = name, colour = colour, cost = cost,
	  pool = pool, isdistributed = false, nbidders = 1, winner = "No Winner",
	  bidders = { [0] = { name = "No Winner" } } 
        }
    --[[{
        link=link,
        name=name,
        colour=colour,
        cost=cost,
        pool=pool,
        nbidders = 1,
        winner = "No Winner",
        bidders = {[0]="No Winner"}
    }]]
    
    self:InitBidLine(self.bidding.nitems)
--    self:Print("Added bidable -- ("..cost.." "..self.poolnames[pool].." pts):"..link)
    self.bidding.nitems = self.bidding.nitems + 1
    self:ShowBidUI()
end

--[[
    DKPmon:AnnounceWinners()
    Description:
        Announces all selected item winners to raid chat
    Input:
        None
    Output:
        None
]]
function DKPmon:AnnounceWinners()
    SendChatMessage("And the winners are:", "RAID")
    local i
    for i = 0, (self.bidding.nitems-1), 1 do
        SendChatMessage(string.format("%s -- %s", self.bidding[i].link, self.bidding[i].winner), "RAID")
    end
    -- Enable the award points button
    self.GUI.bidUI.deductButton:Enable()
    -- Enable the buttons for distributing via master loot (but only if the loot mode is masterlooting)
    local lootmethod, mlID = GetLootMethod()
    if (lootmethod == "master") and (mlID == 0) then
        -- We're in masterlooting mode, and I'm the master looter
        -- So, we can enable the ML buttons
        for i = 0, (self.bidding.nitems-1), 1 do
            self.GUI.bidUI.items[i].ml:Enable()
        end
    end
end

--[[
    DKPmon:OpenBidding()
    Description:
        Opens bidding by:
            1) Set DKPmon.bidding.open = true
            2) Send, to raid chat, one message for each item open for bidding.
               Message format: (<#> <poolname> pnts): [<itemname>]
    Input:
        None
    Output:
        Just to raid chat
]]
function DKPmon:OpenBidding()
    if self.bidding.nitems == 0 then
        self:Print("No items to bid on!")
        return
    end
    self.bidding.bidstate = 1
--    self:Print("Open bidding")
    self.GUI.bidUI.openButton:Disable()
    self.GUI.bidUI.closeBidsButton:Enable()
    --self:SendClaimLeader()
    --self.amLeader = true
    self:SendOpenBidding()
end

--[[
    DKPmon:StartCloseBiddingTimer()
    Description:
        Starts the timer to close bidding, also called when the metronome ticks for the close of bidding
    Input:
        None
    Output:
        Just to raid chat
]]
function DKPmon:StartCloseBiddingTimer()
    local ctime = self.bidding.closeTime
    local timerDur
    --self:Print("Close bidding in "..time.." seconds")
    self:StopMetro("DKPmonClose")
    if ctime == 0 then
        -- Time's up, close the bidding.
        if IsRaidOfficer() then
            SendChatMessage("Bidding is now closed.", "RAID_WARNING")
        else
            SendChatMessage("Bidding is now closed.", "RAID")
        end
        self.bidding.bidstate = 2 -- Move to winner picking
        self:SendCloseBidding()
        self.GUI.bidUI.openButton:Enable()
        self.GUI.bidUI.announceButton:Enable()
        self.GUI.bidUI.closeCancelButton:Hide()
        self.GUI.bidUI.closeBidsButton:Show()
        self.GUI.bidUI.closeBidsButton:Disable()
        return
    end
    if IsRaidOfficer() then
        SendChatMessage("Bidding closing in "..ctime.." seconds.", "RAID_WARNING")
    else
        SendChatMessage("Bidding closing in "..ctime.." seconds.", "RAID")
    end
    if ctime > 10 then
        timerDur = ctime - 10
    else
        if ctime > 5 then
            timerDur = ctime - 5
        else
            timerDur = 1
        end
    end
    self.bidding.closeTime = ctime - timerDur
    self:ChangeMetroRate("DKPmonClose", timerDur)
    self:StartMetro("DKPmonClose")
end

--[[
    DKPmon:CancelCloseTimer()
    Description:
        Called when the "Cancel Close" button is pressed on the GUI
    Input:
        None
    Output:
        Just to raid chat
]]
function DKPmon:CancelCloseTimer()
    self:StopMetro("DKPmonClose")
    self.bidding.closeTime = 0
    SendChatMessage("Close of bidding countdown cancelled", "RAID")
    self.GUI.bidUI.closeCancelButton:Hide()
    self.GUI.bidUI.closeBidsButton:Show()
    self.GUI.bidUI.closeBidsButton:Enable()
end

--[[
    DKPmon:DeductWinnerPts()
    Description:
        Called when the user clicks on the button to deduct points from the item winners.
    Input:
        None
    Output:
        None
]]
function DKPmon:DeductWinnerPts()
    --self:Print("Deducting winner's points")
    if self.db.realm.usezerosum == false then
        -- We're in Points as currency mode.
        -- Pop up the dialog confirming that the user wants to do the points deduction.
        if StaticPopupDialogs["DKPMON_CONFIRM_PACDEDUCT"] == nil then
            StaticPopupDialogs["DKPMON_CONFIRM_PACDEDUCT"] = {
                text = "Points mode: Points as currency.\n\nAre you positive that you wish to deduct points for these items under the points as currency ruleset?",
                button1 = "Yes",
                button2 = "No",
                OnAccept = function()
                    DKPmon:PointsAsCurrencyDeduct()
                end,
                OnCancel = function() end,
                sound = "levelup2",
                timeout = 10,
                whiledead = 1,
                hideOnEscape = 1
            }
        end
        StaticPopup_Show("DKPMON_CONFIRM_PACDEDUCT")
    else
        -- We're in zero-sum mode.
        if StaticPopupDialogs["DKPMON_CONFIRM_ZSDEDUCT"] == nil then
            StaticPopupDialogs["DKPMON_CONFIRM_ZSDEDUCT"] = {
                text = "temp",
                button1 = "Yes",
                button2 = "No",
                OnAccept = function()
                    DKPmon:ZeroSumDeduct()
                end,
                OnCancel = function() end,
                sound = "levelup2",
                timeout = 10,
                whiledead = 1,
                hideOnEscape = 1
            }
        end
        StaticPopupDialogs["DKPMON_CONFIRM_ZSDEDUCT"].text = 
            "Points mode: Zero-sum.\n\nAre you positive that you wish to deduct points for these items under the zero sum ruleset?\n\Reminder: To award the points for these items, you still need to go to the award GUI (/dkp award)."
        StaticPopup_Show("DKPMON_CONFIRM_ZSDEDUCT")
    end
end

--[[
    DKPmon:PointsAsCurrencyDeduct()
    Description:
        Deduct points from the item winners in acordance with standard points as currency DKP.
        Then clear the Bidding Interface window.
    Input:
        None
    Output:
        None
]]
function DKPmon:PointsAsCurrencyDeduct()
    self.bidding.bidstate = 0
    local i = 0
    while i ~= self.bidding.nitems do
        local itemTab = self.bidding[i]
        if itemTab.winner ~= "No Winner" then
            --self:Print(string.format("Deduct from %s: %d from pool %d", self.bidding[i].winner, self.bidding[i].cost, self.bidding[i].pool ))
            -- Find the winning name in the list of bidders for this item so that we can determine
            -- what bidding name to deduct the points from
            local winnername = itemTab.winner
            local j=1
            while (j < itemTab.nbidders) and (itemTab.bidders[j].name ~= winnername) do
                j = j + 1
            end
            if (j == itemTab.nbidders) then
                -- For some reason the winning name wasn't found in the list.
                self:Print("ERROR: Winner "..winnername.." not found in list of bidders for "..itemTab.name)
            else
                winnername = itemTab.bidders[j].bidder
                self:SpendPoints(winnername, itemTab.pool, itemTab.cost)
                self:LogDB_AddItemEvent(itemTab.name, itemTab.cost, itemTab.pool, winnername)
            end
        end
        i = i + 1
    end
    
    self:ConfirmedClearBidItems()
end

--[[
    DKPmon:ZeroSumDeduct()
    Description:
        Deduct points from the item winners in acordance with standard zero-sum DKP.
        Then clear the Bidding Interface window.
    Input:
        None
    Output:
        None
]]
function DKPmon:ZeroSumDeduct()
    self.bidding.bidstate = 0
    -- 1) Count how many players we'll have to split the points among
    local nraid = self:CountRaidMembers()
    local nstandby = self:CountStandbyMembers()
    local ndb = self:CountDBSelected()
    local nplayers = nraid + nstandby + ndb
    -- 2) For each item: (a) Deduct the points from the winner, (b) Add the points div #players to everyone in the list from (1) except the person who won
    local i = 0
    local totalPnts = 0
    while i ~= self.bidding.nitems do
        local itemTab = self.bidding[i]
        if itemTab.winner ~= "No Winner" then
            --self:Print(string.format("Deduct from %s: %d from pool %d", self.bidding[i].winner, self.bidding[i].cost, self.bidding[i].pool ))
            -- Find the winning name in the list of bidders for this item so that we can determine
            -- what bidding name to deduct the points from
            local winnername = itemTab.winner
            local j=1
            while (j < itemTab.nbidders) and (itemTab.bidders[j].name ~= winnername) do
                j = j + 1
            end
            if (j == itemTab.nbidders) then
                -- For some reason the winning name wasn't found in the list.
                self:Print("ERROR: Winner "..winnername.." not found in list of bidders for "..itemTab.name)
            else
                -- Deduct points from the winner.
                winnername = itemTab.bidders[j].bidder
                self:SpendPoints(winnername, itemTab.pool, itemTab.cost)
                self:LogDB_AddItemEvent(itemTab.name, itemTab.cost, itemTab.pool, winnername)                
          end
        end
        -- Add points to the raid.
        if (self.db.realm.unawardedpts[itemTab.pool] == nil) then self.db.realm.unawardedpts[itemTab.pool] = 0 end
        self.db.realm.unawardedpts[itemTab.pool] = self.db.realm.unawardedpts[itemTab.pool] + itemTab.cost
        --[[
        local npnts = itemTab.cost / nplayers
        local playersawarded = {}
        self:AwardCustomToList(self.awards.raid, npnts, itemTab.pool, playersawarded)
        self:AwardCustomToList(self.awards.standby, npnts, itemTab.pool, playersawarded)
        if self.awards.dblist ~= nil then
            self:AwardCustomToList(self.awards.dblist, npnts, itemTab.pool, playersawarded)
        end
        -- Log the points award
        -- Note: AddAwardEvent takes over the playersawarded table, do don't delete/use it anymore
        self:LogDB_AddAwardEvent(string.format("Drop: %s", itemTab.name), npnts, itemTab.pool, playersawarded)
        totalPnts = totalPnts + npnts
  ]]
        i = i + 1
    end

    -- Print out the messages that points were awarded.
    --[[
    local msg = totalPnts.." points awarded."
    if nraid > 0 then SendChatMessage(msg, "RAID") end
    if nstandby > 0 then SendChatMessage(msg, "GUILD") end
    if ndb > 0 then SendChatMessage(msg, "OFFICER") end
    ]]
    self:ConfirmedClearBidItems()
end

--[[
    DKPmon:WinnerClick()
    Description:
        Called when the user clicks on the bidder list for an item
    Input:
        None
    Output:
        Just to raid chat
]]
function DKPmon:WinnerClick(line)
    --self:Print("Click! on line "..line)
    self:TableDelete(dewOptions) -- Recursively reclaim the table
    dewOptions = { type = "group", args = {} }
    dewOptions.args.header = {} 
    dewOptions.args.header.type = "header"
    dewOptions.args.header.name = self.bidding[line].link
    dewOptions.args.header.order = 1
    
    local pos, nametab
    for pos, nametab in pairs(self.bidding[line].bidders) do
        if nametab ~= nil then
            local n = nametab.name
            local key = string.format("%d", pos)
            dewOptions.args[key] = {} 
            local tab = dewOptions.args[key]
            local playerPnts = self:GetPlayerPoints(nametab.bidder, self.bidding[line].pool)
            tab.type = 'execute'
            if nametab.name == "No Winner" then
                tab.name = nametab.name
                tab.order = 2
            else
                if nametab.name == nametab.bidder then
                    -- The sender used his/her own character name to bid
                    tab.name = string.format("|cff%s%s|r(%g)", Bidder.classInfo.classHex[nametab.class], nametab.name, playerPnts)
                else
                    -- The sender used a different character's name to bid
                    tab.name = string.format("|cff%s%s|r[|cff%s%s|r](%g)", Bidder.classInfo.classHex[nametab.class], nametab.name, Bidder.classInfo.classHex[self.bidderClasses[nametab.bidder]], nametab.bidder, playerPnts)
                end
                tab.order = 35000 - playerPnts
            end
            tab.desc = string.format("Select %s to win the item", nametab.name)
            tab.func = function() 
                        DKPmon.bidding[line].winner = n
                        DKPmon:UpdateBidLine(line); 
                       end
        end
    end
    
    dewdrop:Open(self.GUI.bidUI.frame, 'children', dewOptions, 'point', "TOPLEFT", 'relativePoint', "TOPRIGHT")
end

--[[
    DKPmon:MasterLootItem()
    Description:
        Called when the user clicks on button to distribute an item via masterlooter
    Input:
        item: The item number of the item to be distributed
    Output:
        None
]]
function DKPmon:MasterLootItem(item)
    local iTab = self.bidding[item]
    if iTab.isdistributed then return end -- Item's already been given out
    if iTab.winner == "No Winner" then return end -- Item doesn't have a winner
    -- 1) Determine the slot number of the item 
    local nItems = GetNumLootItems()
    local i = 0
    local link
    while not (i > nItems) and (link ~= iTab.link) do
        i = i + 1
        link = GetLootSlotLink(i)
    end
    if (link ~= iTab.link) then
        -- Couldn't find the item, for some reason
        self:Print(string.format("ERROR: Could not find %s in loot window.", iTab.link))
        return
    end
    local slot = i
    -- 2) Determine the candidate index of the winner.
    LootFrame_OnEvent("UPDATE_MASTER_LOOT_LIST")
    local nRaiders = GetNumRaidMembers()
    local name
    i = 0
    while not (i > nRaiders) and (name ~= iTab.winner) do
        i = i + 1
        name = GetMasterLootCandidate(i)
    end
    if (name ~= iTab.winner) then
        -- Couldn't find the winner in the list of raid members eligable to receive loot
        self:Print(string.format("ERROR: %s could not be found in the MasterLooter list for %s", iTab.winner, iTab.link))
        return
    end
    local candidate = i
    GiveMasterLoot(slot, candidate)
end