﻿--[[
    Copyright Daniel D. Neilson, 2006

    This file is part of DKPmon.

    DKPmon is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DKPmon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DKPmon; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
]]
--[[
    Functions to create and manipulate the GUI element for bid 
    management.
    
    Data fields for this UI are:
    
    DKPmon.GUI.bidUI.frame -- The bidding interface frame
    DKPmon.GUI.bidUI.openButton -- The button for openning bidding
    DKPmon.GUI.bidUI.timeSlider -- Slider for selecting the time 'til bidding closes
    DKPmon.GUI.bidUI.sliderTxt -- Text display for timeSlider
    DKPmon.GUI.bidUI.closeBidsButton -- Button for closing bidding
    DKPmon.GUI.bidUI.closeCancelButton -- Button for cancelling the close-bidding countdown
    DKPmon.GUI.bidUI.announceButton -- Button for announcing bid winners
    DKPmon.GUI.bidUI.deductButton -- Button for deducting points from the winners
    DKPmon.GUI.bidUI.items -- Table of 10 widget-tables for each bid item
            [0..9] = { ml -- Button for Masterlooting this item to the winner
                       item -- Text field for the item name
                       num  -- Text field for the # of bidders
                       frame -- Frame for registering clicks on the bidder list
                       winner -- Text field for the selected item winner
                     }
]]

local function CreateButton(parent, width, height, text, xoffs, yoffs)
    local b = CreateFrame("Button", nil, parent, "UIPanelButtonTemplate")
    b:SetWidth(width); b:SetHeight(height)
    b:SetText(text)
    if xoffs ~= nil and yoffs ~= nil then
        b:SetPoint("TOPLEFT", parent, "TOPLEFT", xoffs, yoffs)
    end
    return b
end

local function CreateItemLine(parent, line)
    local tab = {}
    local f,t
    local spacing=25
    -- Button for masterlooting the item to the winner
    t = CreateFrame("Button", nil, parent, "UIPanelButtonTemplate")
    t:SetWidth(20); t:SetHeight(22)
    t:SetText("ML")
    t:SetPoint("TOPLEFT", parent, "TOPLEFT", 10, -77-line*spacing)
    t:SetScript("OnClick", function() DKPmon:MasterLootItem(line) end)
    tab.ml = t
    -- Create the FontString for the item name
    t = parent:CreateFontString()
    t:SetPoint("TOPLEFT", parent, "TOPLEFT", 30, -80-line*spacing)
    t:SetFontObject(GameFontNormal); t:SetWidth(280)
    t:SetText("")
    t:SetJustifyV("MIDDLE")
    tab.item = t
    -- Create the FontString for the number of bidders
    t = parent:CreateFontString()
    t:SetPoint("TOPLEFT", parent, "TOPLEFT", 310, -80-line*spacing)
    t:SetFontObject(GameFontNormal); t:SetWidth(30)
    t:SetText("")
    t:SetJustifyV("MIDDLE")
    tab.num = t;
    -- Create the fontstring & clickable frame for the winner name
    f = CreateFrame("Frame", nil, parent)
    f:SetWidth(130); f:SetHeight(spacing-5);
    f:EnableMouse(true)
    f:SetPoint("TOPLEFT", parent, "TOPLEFT", 340, -80-line*spacing)
    f:SetScript("OnMouseDown", function() DKPmon:WinnerClick(line); end)
    tab.frame = f
    t = f:CreateFontString()
    t:SetPoint("TOPLEFT", parent, "TOPLEFT", 340, -80-line*spacing);  
    t:SetFontObject(GameFontNormal); t:SetWidth(130)
    t:SetText("")
    t:SetJustifyV("MIDDLE")
    tab.winner = t
    
    tab.frame:Hide()
    tab.item:Hide()
    tab.num:Hide()
    
    return tab
end

function DKPmon:CreateBidFrame()
    --self:Print("Creating frame");
    DKPmon.GUI.bidUI = {}
    local frame = CreateFrame("Frame", "DKPmonBidFrame", UIParent)
    frame:SetWidth(480)
    frame:SetHeight(360)
    frame:SetBackdrop( {
            bgFile = "Interface/Tooltips/UI-Tooltip-Background",
            edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
            tile = true, tileSize=16, edgeSize=16,
            insets = {left=4, right=4, top=4, bottom=4}
        } )
    frame:SetBackdropColor(0.16, 0.28, 0.61, 0.9) -- Blueish
    --frame:SetBackdropColor(0.29, 0.72, 0.70, 0.9) --Cyan
    --frame:SetBackdropColor(0.89, 0.15, 0.22, 0.9) -- Red
    frame:EnableMouse(true)
    frame:SetMovable(true)
    frame:SetPoint("CENTER", 0, 0)
    frame:CreateTitleRegion()
    local tR= frame:GetTitleRegion()
    tR:SetWidth(150); tR:SetHeight(20)
    tR:SetPoint("CENTER", frame, "TOP", 0, -10)

    -- Add the title to the top of the window
    local t = frame:CreateFontString()
    t:SetAllPoints(tR)
    t:SetTextColor(1.0, 0.819, 0.0);
    t:SetWidth(140)
    t:SetFontObject(QuestTitleFont)
    t:SetText("Bidding Interface")
    
    -- Add the "close window" button
    local b = CreateFrame("Button", nil, frame, "UIPanelCloseButton")
    b:SetPoint("CENTER", frame, "TOPRIGHT", -16, -16)
    b:Show()
    
    -- Add the "Open Bidding" button
    b = CreateButton(frame, 100, 22, "Open Bidding", 15, -30)
    b:SetScript("OnClick", function() DKPmon:OpenBidding() end)
    self.GUI.bidUI.openButton = b
        
    -- Add the close-time slider
    b = CreateFrame("Slider", nil, frame)
    b:SetOrientation("HORIZONTAL")
    b:SetMinMaxValues(5, 60); b:SetValueStep(5); b:SetValue(30)
    b:SetWidth(100); b:SetHeight(22)
    b:SetPoint("TOPLEFT", frame, "TOPLEFT", 130, -30)
    b:SetBackdrop({
			bgFile="Interface\\Buttons\\UI-SliderBar-Background",
			edgeFile="Interface\\Buttons\\UI-SliderBar-Border",
			tile=true,edgeSize=8,tileSize=8,
			insets={left=3,right=3,top=3,bottom=3} } )
    b:CreateTexture()
    b:SetThumbTexture("Interface\\Buttons\\UI-SliderBar-Button-Horizontal")
    t = b:CreateFontString(nil, "ARTWORK")
    t:SetFontObject(GameFontGreenSmall)
    --t:SetHeight(8);
    t:SetText("30s")
	t:SetPoint("LEFT", b, "RIGHT")
    b:SetScript("OnValueChanged", function() DKPmon.GUI.bidUI.sliderTxt:SetText(this:GetValue().."s")end)
    self.GUI.bidUI.timeSlider = b
    self.GUI.bidUI.sliderTxt = t
    
    -- Add the "Close Bidding" button
    b = CreateButton(frame, 100, 22, "Close Bidding", 260, -30)
    b:SetScript("OnClick", function() 
        DKPmon.bidding.closeTime = DKPmon.GUI.bidUI.timeSlider:GetValue()
        DKPmon.GUI.bidUI.closeBidsButton:Hide()
        DKPmon.GUI.bidUI.closeCancelButton:Show()
        DKPmon:StartCloseBiddingTimer() 
    end)
    self.GUI.bidUI.closeBidsButton = b
    b = CreateButton(frame, 100, 22, "Cancel Close", 260, -30)
    b:SetScript("OnClick", function() 
        DKPmon:CancelCloseTimer()
    end)
    self.GUI.bidUI.closeCancelButton = b
    self.GUI.bidUI.closeCancelButton:Hide()
    
    -- Add the "Announce" button
    b = CreateButton(frame, 80, 22, "Announce", 360, -30)
    b:SetScript("OnClick", function() DKPmon:AnnounceWinners() end)
    self.GUI.bidUI.announceButton = b
    
    -- Add the "Clear All" button
    b = CreateButton(frame, 100, 22, "Clear All", 350, -330)
    b:SetScript("OnClick", function() DKPmon:ClearBidItems() end)
    
    -- Add the "Deduct Winner Points" button
    b = CreateButton(frame, 150, 22, "Deduct Winner Points", 40, -330)
    b:SetScript("OnClick", function() DKPmon:DeductWinnerPts() end)
    self.GUI.bidUI.deductButton = b
    
    -- Add the column labels: Items, #, Winner
    t = frame:CreateFontString()
    t:SetPoint("TOPLEFT", frame, "TOPLEFT", 10,-60)
    t:SetFontObject(GameFontWhite); t:SetWidth(300)
    t:SetText("Biddable Items")

    t = frame:CreateFontString()
    t:SetPoint("TOPLEFT", frame, "TOPLEFT", 300,-60)
    t:SetFontObject(GameFontWhite); t:SetWidth(50)
    t:SetText("#bids")

    t = frame:CreateFontString()
    t:SetPoint("TOPLEFT", frame, "TOPLEFT", 340,-60)
    t:SetFontObject(GameFontWhite); t:SetWidth(130)
    t:SetText("Winner")
    
    -- Create the 10 item bid lines.
    DKPmon.GUI.bidUI.items = {
        [0]=CreateItemLine(frame, 0),
        [1]=CreateItemLine(frame, 1),
        [2]=CreateItemLine(frame, 2),
        [3]=CreateItemLine(frame, 3),
        [4]=CreateItemLine(frame, 4),
        [5]=CreateItemLine(frame, 5),
        [6]=CreateItemLine(frame, 6),
        [7]=CreateItemLine(frame, 7),
        [8]=CreateItemLine(frame, 8),
        [9]=CreateItemLine(frame, 9)        
    }
    
    -- Create the tablet that'll contain the bidder list when the user clicks
    local dewdrop = AceLibrary("Dewdrop-2.0")
    dewdrop:Register(frame, { "dontHook", true })
    
 --[[
    -- Draw the measuring stick
    local x = 10
    while x ~= 480 do
        t = frame:CreateFontString()
        t:SetPoint("TOPLEFT", frame, "TOPLEFT", x,-80)
        t:SetFontObject(GameFontWhite); t:SetWidth(5)
        t:SetJustifyH("LEFT")
        if math.mod(x, 20) == 0 then
            t:SetText("|")
        else
            t:SetText(".")
        end
        x = x + 10
    end
]]
    self.GUI.bidUI.frame = frame
    self:ResetBidUI()
    frame:Hide()
end

function DKPmon:CreateConfirmClearDialog()
    StaticPopupDialogs["DKPMON_CONFIRM_CLEAR"] = {
        text = "Bidding is active, are you sure you want to clear all?",
        button1 = "Yes",
        button2 = "No",
        OnAccept = function()
            DKPmon:ConfirmedClearBidItems()
        end,
        OnCancel = function() end,
        sound = "levelup2",
        timeout = 10,
        whiledead = 1,
        hideOnEscape = 1
    }
end

function DKPmon:ShowBidUI()
    if self.GUI.bidUI.frame == nil then self:CreateBidFrame(); end
    self.GUI.bidUI.frame:Show()
end

function DKPmon:InitBidLine(line)
    if self.GUI.bidUI.frame == nil then self:CreateBidFrame(); end
    local tab = self.GUI.bidUI.items[line]
    tab.ml:Disable()
    tab.winner:SetText(self.bidding[line].winner)
    tab.num:SetText(self.bidding[line].nbidders-1)
    tab.item:SetText("("..self.bidding[line].cost..")|c"..self.bidding[line].colour.."["..self.bidding[line].name.."]|r")
    tab.ml:Show()
    tab.frame:Show()
    tab.item:Show()
    tab.num:Show()
end

function DKPmon:UpdateBidLine(line)
    local tab = self.GUI.bidUI.items[line]    
    tab.winner:SetText(self.bidding[line].winner)
    tab.num:SetText(self.bidding[line].nbidders-1)
end

function DKPmon:ResetBidUI()
    if self.GUI.bidUI.frame == nil then return end
    self.GUI.bidUI.openButton:Enable()
    self.GUI.bidUI.closeBidsButton:Disable()
    self.GUI.bidUI.closeBidsButton:Show()
    self.GUI.bidUI.closeCancelButton:Hide()
    self.GUI.bidUI.announceButton:Disable()
    self.GUI.bidUI.deductButton:Disable()
    
    local i=0
    while i ~= self.bidding.nitems do
        self:InitBidLine(i)
        i = i + 1
    end
    while i ~= 10 do
        local tab = self.GUI.bidUI.items[i]
        tab.ml:Hide()
        tab.frame:Hide()
        tab.item:Hide()
        tab.num:Hide()
        i = i + 1
    end
end