﻿--[[
    Copyright Daniel D. Neilson, 2006

    This file is part of DKPmon.

    DKPmon is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DKPmon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DKPmon; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
]]

local options = {
    type = 'group',
    args = {
        biditem = {
            type = 'text',
            name = 'biditem',
            desc = 'Supply a list of item links to bid on.',
            usage = '<itemlink>[<itemlink>]+',
            get = false,
            set = "ParseBidItems",
            input = false
        },
        showbids = {
            type = 'execute',
            name = 'showbids',
            desc = 'Show the bidding interface.',
            func = function() DKPmon:ShowBidUI() end
        },
        leader = {
            type = 'execute',
            name = 'leader',
            desc = 'Claim the DKPmon leader role.',
            func = function() DKPmon.amLeader = true; DKPmon:SendClaimLeader() end
        },
        award = {
            type = 'execute',
            name = 'award',
            desc = 'Open the points awarding GUI.',
            func = function() DKPmon:ShowAwardUI() end
        },
        bcast = {
            type = 'execute',
            name = 'bcast',
            desc = 'Broadcast your points database to the other DKPmon in the raid.',
            func = function() DKPmon:BroadcastPointsDB() end
        },
        sync = {
            type = 'execute',
            name = 'sync',
            desc = 'Broadcast a points database sync request',
            func = function() DKPmon:ConfirmSyncRequest() end
        },
        bidderinstall = {
            type = 'execute',
            name = 'bidderinstall',
            desc = 'Initiate a query to check who in the raid does not have bidder installed.',
            func = function() DKPmon:InitiateBidderInstallCheck() end
        },
        dump = {
            type = 'execute',
            name = 'dump',
            desc = 'Print entire player points database to a window for cut and paste.',
            func = function() DKPmon:DumpDBToWindow() end
        },
        password = {
            type = 'text',
            name = 'password',
            desc = 'Set your broadcast password.',
            usage = '<password>',
            get = function() return DKPmon.db.realm.password end,
            set = function(v) DKPmon.db.realm.password = v end,
            input = false
        },
        autoloot = {
            type = 'toggle',
            name = 'autoloot',
            desc = 'Enable automatic population of the loot list on looting events.',
            get = function() return DKPmon.db.realm.autoloot end,
            set = function(v) DKPmon.db.realm.autoloot = v end
        },
        zerosum = {
            type = 'toggle',
            name = 'zerosum',
            desc = 'Toggle zero sum DKP mode.',
            get = function() return DKPmon.db.realm.usezerosum end,
            set = function(v) DKPmon.db.realm.usezerosum = v; DKPmon:RefreshAwardUI() end
        }
    }
}

-- Set the console commands
DKPmon:RegisterChatCommand({"/dkp", "/dkpmon"}, options)

local function CreateConfirmBCastDialog()
    StaticPopupDialogs["DKPMON_CONFIRM_BROADCAST"] = {
        text = "Are you sure you want to broadcast your points database to the raid?",
        button1 = "Yes",
        button2 = "No",
        OnAccept = function()
            DKPmon:SendBroadcastPointsDB()
        end,
        OnCancel = function() end,
        sound = "levelup2",
        timeout = 10,
        whiledead = 1,
        hideOnEscape = 1
    }
end
function DKPmon:BroadcastPointsDB()
   if StaticPopupDialogs["DKPMON_CONFIRM_BROADCAST"] == nil then
        CreateConfirmBCastDialog()
    end
    StaticPopup_Show("DKPMON_CONFIRM_BROADCAST")
end

local function CreateConfirmSyncDialog()
    StaticPopupDialogs["DKPMON_CONFIRM_SYNC"] = {
        text = "Are you sure you want to initiate a sync?",
        button1 = "Yes",
        button2 = "No",
        OnAccept = function()
            DKPmon:SendSyncRequest()
        end,
        OnCancel = function() end,
        sound = "levelup2",
        timeout = 10,
        whiledead = 1,
        hideOnEscape = 1
    }
end
function DKPmon:ConfirmSyncRequest()
    if StaticPopupDialogs["DKPMON_CONFIRM_SYNC"] == nil then
        CreateConfirmSyncDialog()
    end
    StaticPopup_Show("DKPMON_CONFIRM_SYNC")
end


function DKPmon:InitiateBidderInstallCheck()
    self:ChangeMetroRate("DKPmonBidderCheck", 10.0)
    self:StartMetro("DKPmonBidderCheck")
    self:SendBidderInstallCheck()
end

function DKPmon:PrintNoBidderInstalledList()
    self:StopMetro("DKPmonBidderCheck")
    self:Print("The following people do not have Bidder installed:")
    local nbad = 0
    local name, inst
    for name, inst in pairs(self.raidList) do
        if inst == 0 then
            self:Print(name)
            nbad = nbad + 1
        end
    end
    if nbad == 0 then self:Print("None (yay!)") end
end


function DKPmon:DumpDBToWindow()
    if self.GUI.dumpUI.frame == nil then
        local frame = CreateFrame("Frame", "DKPmonDumpFrame", UIParent)
        frame:SetWidth(300)
        frame:SetHeight(60)
        frame:SetBackdrop( {
                bgFile = "Interface/Tooltips/UI-Tooltip-Background",
                edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
                tile = true, tileSize=16, edgeSize=16,
                insets = {left=4, right=4, top=4, bottom=4}
            } )
        frame:SetBackdropColor(0.16, 0.28, 0.61, 0.9) -- Blueish
        --frame:SetBackdropColor(0.29, 0.72, 0.70, 0.9) --Cyan
        --frame:SetBackdropColor(0.89, 0.15, 0.22, 0.9) -- Red
        frame:EnableMouse(true)
        frame:SetMovable(true)
        frame:SetPoint("CENTER", 0, 0)
        frame:CreateTitleRegion()
        local tR= frame:GetTitleRegion()
        tR:SetWidth(150); tR:SetHeight(20)
        tR:SetPoint("CENTER", frame, "TOP", 0, -10)

        -- Add the title to the top of the window
        t = frame:CreateFontString()
        t:SetAllPoints(tR)
        t:SetTextColor(1.0, 0.819, 0.0);
        t:SetWidth(140)
        t:SetFontObject(QuestTitleFont)
        t:SetText("Cut & Paste")
        
        -- Add the "close window" button
        b = CreateFrame("Button", nil, frame, "UIPanelCloseButton")
        b:SetPoint("CENTER", frame, "TOPRIGHT", -16, -16)
        
        -- Put a big ol' editbox to fill the frame

        local f = CreateFrame("EditBox", nil, frame, "InputBoxTemplate")
        --f:SetMultiLine(true)
        f:SetHeight(30); f:SetWidth(280)
        f:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 10, 10)
        f:SetAutoFocus(false)

        self.GUI.dumpUI.frame = frame
        self.GUI.dumpUI.editbox = f
        frame:Hide()
    end
    
    -- Okay, generate the giant text string/table that we'll put in the text box.
    local pntstr = ""
    local name, nametab
    local maxpool = table.getn(self.poolnames)
    for name, nametab in pairs(self.db.realm.points) do
        pntstr = string.format("%s%s:%s", pntstr, name, nametab.info.class)
        local pool = 0
        for pool = 0,maxpool,1 do
            pntstr = pntstr..":"..self:GetPlayerPoints(name, pool)
        end
        pntstr = string.format("%s\n", pntstr)
    end
    self.GUI.dumpUI.editbox:SetMaxLetters(string.len(pntstr)+1)
    self.GUI.dumpUI.editbox:SetText(pntstr)
    self.GUI.dumpUI.editbox:HighlightText()
    self.GUI.dumpUI.frame:Show()
end