﻿--[[
    Copyright Daniel D. Neilson, 2006

    This file is part of DKPmon.

    DKPmon is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DKPmon is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DKPmon; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
]]
--[[
    Functions for manipulating the player-points database.
]]

--[[
    DKPmon:GetPlayerPoints(name, pool)
    Description:
        Fetches the number of <pool#> points that player "name" has.
    Input:
        name: The bidname to lookup
        pool: The pool number to lookup
    Output:
        integer: The total number of points the player has in the pool
]]
function DKPmon:GetPlayerPoints(name, pool)
    local tab = self.db.realm.points[name]
    if tab == nil then return 0; end
    tab = tab.points
    if tab[pool] == nil then return 0; end
    return tab[pool].earned - tab[pool].spent - tab[pool].lost;
end

--[[
]]
function DKPmon:AddDBEntry(name, class)
    self.db.realm.points[name] = {}
    self.db.realm.points[name].info = {}
    self.db.realm.points[name].info.class = class
    self.db.realm.points[name].points = {}
    
    self.awards.ndbselected = 0
    if self.awards.dblist ~= nil then
        self:TableDelete(self.awards.dblist)
        self.awards.dblist = nil
    end
end

--[[
    DKPmon:AwardPoints(name, pool, pnts)
    Description:
        Awards points to the name given
    Input:
        name: The bidname to award points to
        class: The class of the bidname
        pool: The pool number to award points in
        pnts: The number of points to add.
    Output:
        None
]]
function DKPmon:AwardPoints(name, pool, pnts)
    -- If the name's not in the DB, then add it.
    if self.db.realm.points[name] == nil then 
        self:AddDBEntry(name, self.bidderClasses[name])
    end
    if self.bidderClasses[name] ~= nil then
        self.db.realm.points[name].info.class = self.bidderClasses[name] -- We might be from an older database that didn't fill this field in.
    end
    local tab = self.db.realm.points[name].points
    if tab[pool] == nil then
        tab[pool] = { earned = 0, spent = 0, lost = 0 }
    end
    tab[pool].earned = tab[pool].earned + pnts 
end

--[[
    DKPmon:SpendPoints(name, pool, pnts)
    Description:
        Spends points from the name given
    Input:
        name: The bidname to spend points for
        class: The class of the bidname
        pool: The pool number to spend points in
        pnts: The number of points to spend
    Output:
        None
]]
function DKPmon:SpendPoints(name, pool, pnts)
    -- If the name's not in the DB, then add it.
    if self.db.realm.points[name] == nil then 
        self:AddDBEntry(name, self.bidderClasses[name])
    end
    if self.bidderClasses[name] ~= nil then
        self.db.realm.points[name].info.class = self.bidderClasses[name] -- We might be from an older database that didn't fill this field in.
    end
    local tab = self.db.realm.points[name].points
    if tab[pool] == nil then
        tab[pool] = { earned = 0, spent = 0, lost = 0 }
    end
    tab[pool].spent = tab[pool].spent + pnts 
end

--[[
    DKPmon:LosePoints(name, pool, pnts)
    Description:
        Deducts/penalizes points from the name given
    Input:
        name: The bidname to deduct
        class: The class of the bidname
        pool: The pool number to deduct from
        pnts: The number of points to deduct
    Output:
        None
]]
function DKPmon:LosePoints(name, pool, pnts)
    -- If the name's not in the DB, then add it.
    if self.db.realm.points[name] == nil then 
        self:AddDBEntry(name, self.bidderClasses[name])
    end
    if self.bidderClasses[name] ~= nil then
        self.db.realm.points[name].info.class = self.bidderClasses[name] -- We might be from an older database that didn't fill this field in.
    end

    local tab = self.db.realm.points[name].points
    if tab[pool] == nil then
        tab[pool] = { earned = 0, spent = 0, lost = 0 }
     end
    tab[pool].lost = tab[pool].lost + pnts 
end

--[[
    DKPmon:UpdatePoints(name, pool, earned, spent, lost)
    Description:
        Set the values in self.db.realm.points[name][pool] to the max of their current values and the values passed.
    Input:
        name: The bidname to deduct
        pool: The pool number to deduct from
        pnts: The number of points to deduct
    Output:
        None
]]
function DKPmon:UpdatePoints(name, pool, earned, spent, lost)
    --self:Print(string.format("UpdatePoints: %s  %d(%d:%d:%d)", name, pool, earned, spent, lost))
    if self.db.realm.points[name] == nil then 
        self:AddDBEntry(name, self.bidderClasses[name])
    end
    if self.bidderClasses[name] ~= nil then
        self.db.realm.points[name].info.class = self.bidderClasses[name] -- We might be from an older database that didn't fill this field in.
    end
    local tab = self.db.realm.points[name].points
    if tab[pool] == nil then
        tab[pool] = { earned = 0, spent = 0, lost = 0 }
    end
    tab[pool].earned = math.max(tab[pool].earned, earned)
    tab[pool].spent = math.max(tab[pool].spent, spent)
    tab[pool].lost = math.max(tab[pool].lost, lost)
end

--[[
    DKPmon:GetBossTable()
    Description:
        A recursive search through self.bossvalues
    Input:
        name: The bidname to deduct
        pool: The pool number to deduct from
        pnts: The number of points to deduct
    Output:
        None
]]
function DKPmon:GetBossTable(bossname, bosstab)
    if bosstab == nil then bosstab = self.bossvalues end
    local name, tab
    for name, tab in pairs(bosstab) do
        if bosstab[name][0] == nil then -- Actual boss names will have a [0] table, the categories will not.
            local ret = self:GetBossTable(bossname, bosstab[name])
            if ret ~= nil then return ret end
        else
            if name == bossname then return tab end
        end
    end
    return nil
end


--[[
    DKPmon:MigrateDB()
    Description:
        Migrates the player points database from whatever version is currently stored
        to the newest format.
    Input:
        None
    Output:
        None
]]
function DKPmon:MigrateDB()
    local oldver = self.db.realm.dbversion
    if oldver == 1.0 then
        -- Update from v1.0 to v1.1
        --[[ 
            Player database is stored as:
                .points["<bidname>"][<pool#>] = ...
            New format:
                .points["<bidname>"] = {
                    ["Info"] = { ["class"] = <string: bidder's classname in english> },
                    ["points"][<pool#>] = ...
        ]]
        local name, tab
        for name, tab in pairs(self.db.realm.points) do
            self.db.realm.points[name] = {}
            self.db.realm.points[name].points = tab
            self.db.realm.points[name].info = {}
            self.db.realm.points[name].info.class = "UNKNOWN"
        end
        oldver = 1.1
    end
    if oldver == 1.1 then
        -- Nothing to do to update from 1.1 to 1.2. 
        -- 1.2 just added the .log field.
        oldver = 1.2
    end
end